package com.example.user.aop;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.Arrays;

/**
 * 简单AOP，示例拦截相应的方法
 */
@Component
@Slf4j
@Aspect  //【AOP】2、定义AOP类
public class SimpleAOP {
    @Autowired
    ObjectMapper objectMapper;
    /**
     * 【AOP】3、定义切入点，对UserService.getById的所有方法进行拦截
     */
    @Pointcut("execution(* com.example.user.controller.UserController.getById(..))")
    public void pointcut() {

    }

    /**
     * 【AOP】4、拦截逻辑，前置通知，方法调用前执行
     * @param joinPoint
     */
    @Before("pointcut()")
    public void onBefore(JoinPoint joinPoint) {
        log.info("【AOP-SimpleAOP】Before：" + joinPoint.getSignature().getName() + "方法开始执行，传入参数："
                + Arrays.asList(joinPoint.getArgs()));
    }

    /**
     * 【AOP】4、拦截逻辑，后置通知，方法调用后执行，不管是否成功
     * @param joinPoint
     */
    @After("pointcut()")
    public void onAfter(JoinPoint joinPoint) {
        log.info("【AOP-SimpleAOP】After：" + joinPoint.getSignature().getName() + "方法结束执行，传入参数："
                + Arrays.asList(joinPoint.getArgs()));
    }

    /**
     * 【AOP】4、拦截逻辑，后置返回通知，如果有正确返回后执行
     * @param joinPoint
     * @param result
     */
    @AfterReturning(value = "pointcut()", returning = "result")
    public void afterReturning(JoinPoint joinPoint, Object result) throws JsonProcessingException {
        String strResult =  objectMapper.writeValueAsString(result);
        log.info("【AOP-SimpleAOP】AfterReturning：" + joinPoint.getSignature().getName() + "方法执行成功返回，传入参数："
                + Arrays.asList(joinPoint.getArgs()) + "，返回结果：" + strResult);
    }

    /**
     * 【AOP】4、拦截逻辑，后置异常通知，如果异常返回后执行
     * @param joinPoint
     * @param exception
     */
    @AfterThrowing(value = "pointcut()", throwing = "exception")
    public void afterThrowing(JoinPoint joinPoint, Exception exception) {
        log.info("【AOP-SimpleAOP】AfterThrowing：" + joinPoint.getSignature().getName() + "方法执行出错，传入参数："
                + Arrays.asList(joinPoint.getArgs()) + "，异常：" + exception);
    }

    /**
     * 【AOP】4、拦截逻辑，环绕
     * @param proceedingJoinPoint
     * @return
     * @throws Throwable
     */
    @Around("pointcut()")
    public Object around(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        Object result = null;

        //获取拦截的方法
        MethodSignature signature = (MethodSignature) proceedingJoinPoint.getSignature();
        Method method = signature.getMethod();

        log.info("【AOP-SimpleAOP】Around（执行方法前）："+method.getName()+ "，传入参数："
                + Arrays.asList(proceedingJoinPoint.getArgs()));

        //拦截方法调用，此处可做一定的逻辑，如果是对权限进行过滤，没有权限不可调用，可return null
        result =  proceedingJoinPoint.proceed();

        String strResult =  objectMapper.writeValueAsString(result);
        log.info("【AOP-SimpleAOP】Around（执行方法后）："+method.getName()+ "，传入参数："
                + Arrays.asList(proceedingJoinPoint.getArgs()) +"，返回结果：" +strResult);

        return result;
    }
}
